#ifndef CONFIG_H
#define CONFIG_H

#define SWVERSION "RETR3cfX 5-19,cf, A&C support "

#include "ABTypes.h"
#include "si5351.h" //  to get SI5351_CRYSTAL_LOAD_C names

//***************   The following values may be PCB specific  *************************************************************************
const uint16_t DEFAULT_DACSETTING = 2048;            //  initial nominal dacsetting at TNOMINAL for DAC.
const uint8_t SI5351_CRYSTAL_LOAD_C = SI5351_CRYSTAL_LOAD_8PF ; //  0,6,8,10 pf are candidates
/* MCP47x6 package markings (Address 0xA0=I2C 0x60 conflicts with Si5351 !!! ) are:
// MCP4706 8 bits  is DB/DC/DD/DE for A0/A1/A2/A3F
// MCP4716 10 bits is DF/DG/DH/DJ for A0/A1/A2/A3
// MCP4726 12 bits is DK/DL/DM/DP for A0/A1/A2/A3
*/
//const byte MCP47x6_defaultaddr = 0x61;            //  for 0xA1  part suffix from n6gn private stock
  const byte MCP47x6_defaultaddr = 0x62;            //  Depends upon part availablity
  
//  Change these depending on the PCB Application 
const boolean COMPILED_FOR_REFERENCE = true;     //  false
const boolean TRANSVERTER = false;
//const boolean COMPILED_FOR_REFERENCE = false;  //  false
//const boolean TRANSVERTER = true;              //  true

const boolean EXTIN = false;                     // True for Si5351 version C using canned VCXO
const uint8_t MAX_CLKS = 3;                      // 8 for Si5351 version C, 3 for version A
//if FE
//const boolean COMPILED_FOR_REFERENCE = false;
//const boolean TRANSVERTER = false;

/************************************   The following are global CONSTANT values that don't change (much) with particular HW ************************************/
/*********************************************************************************************************************************************************/
// Don't put any methods or variable here. These const are in lieu of #defines because it's safer
const int WRITTEN_SIGNATURE = 0xBEEFDEED;   // These are for activestate stored in EEPROM
const int START_ADDRESS     = 0;
 
//  Measured with a DVM
const float VDD = 3.299;            //  IoT p17  3.3V Analog supply  (measured with DVM)const uint8_t  PIN_MOSI = 11;       //  IoT33 Pin for SPI MOSI
const float AREF = 4.024;           //  LP2985 reference output (MEASURED with DVM), used by DAC, maybe ADC, and CP
 
const float DACV_PER_BIT = AREF/4096;
 
const uint8_t  MISO_PIN = 12;       //  IoT33 Pin for SPI MISO (unused)
const uint8_t  SCK_PIN =  13;       //  IoT33 Pin for SPI CLK
 
const int THERMOMETER_PIN = A0;     //  Analog Input from TMP236A4
const int VTUNE_PIN = A1;           //  Analog Input unused on some PCBs
const int VSET_PIN = A2;            //  Analog Input from LP2985 reference, may be 3.6 or 4 VDC depending on parts availability
const int TPDETECT = A3;            //  Filtered Analog of GNSS timepulse, used for GNSS Satellite LOCK DETECT in GNSSTime
const int SDA_PIN = A4;             //
const int SC_PIN = A5;              //
const int Unused_PIN = A6;          //
const int PTT_PIN = A7;             //  Analog Input from PTT
 
const uint8_t TX_EN = 1;              //  MC23008 connections
const uint8_t CONVERTER_BYPASSED = 2; // HF Antenna Engaged when true
const uint8_t CONVERTER_ACTIVE = 3;   // Converter Engaged when true
const uint8_t PREAMPB = 4;
const uint8_t PREAMPA = 5;
const uint8_t TRANSCEIVE = 7;
const uint8_t NOTTRANSCEIVE = 6;
 
const uint32_t freq1k =    1000UL;             // human readable values where ULs are needed for setting things.
const uint32_t freq10k =   10000UL;
const uint32_t freq40k =   40000UL;
const uint32_t freq100k =  100000UL;
const uint32_t freq250k =  250000UL;
const uint32_t freq500k =  250000UL;
const uint32_t freq1M =    1000000UL;
const uint32_t UBLOX_MAGIC_FREQ = 4 * 1171875UL;   // or 4687.500 kHz,   10^4 * (30.72^6/2^16) = 5^4 * (30.72^6/2^12) & freq600/128
const uint32_t freq2M =    2000000UL;
const uint32_t freq5M =    5000000UL;
const uint32_t freq8M =    8000000UL;
const uint32_t freq10M =   10000000UL;
const uint32_t freq15p6M = 15600000UL;
const uint32_t freq20M =   20000000UL;
const uint32_t freq22M =   22625000UL;
const uint32_t freq25M =   25000000UL;
const uint32_t freq27M =   27000000UL;
const uint32_t freq30M =   30000000UL;
const uint32_t freq40M =   40000000UL;
const uint32_t freq41M =   41344000UL;
const uint32_t freq50M =   50000000UL;
const uint32_t freq66M =   66672000UL;
const uint32_t freq80M =   80000000UL;
const uint32_t freq100M =  100000000UL;
const uint32_t freq125M =  125000000UL;
const uint32_t freq135M =  135000000UL;
const uint32_t freq600M =  600000000UL;
const uint32_t freq750M =  750000000UL;
const uint32_t freq800M =  800000000UL;
const uint32_t freq900M =  900000000UL;

const uint32_t UWVREFERENCE = freq25M;
const uint32_t DEFAULT_LO1FREQ = 2625000000UL;      //  Ftune + 2185M (LO2-(LO3FREQ + IF3midband) at startup
const uint32_t LO2FREQ = 2352000000UL;
const uint32_t LO3FREQ = LO2FREQ/16;
 
const uint8_t Synth1_PIN_CE = 2 ;           // LO1 Chip Enable
const uint8_t Synth2_PIN_CE = 3 ;           // Synth2 Chip Enable

const uint8_t Synth1_PIN_LOCK = 8 ;         // LO1 Lock Detect, but make it available, LOW_WiFi1 on RE2
const uint8_t LOW_WiFi1 = 8 ;               // LO1 Lock Detect, but make it available, LOW_WiFi1 on RE2

const uint8_t Synth2_PIN_LOCK = 9 ;         // Synth2 lock Detect, LOW_WiFi2 on RE2
const uint8_t LOW_WiFi2 = 9 ;               // Synth2 lock Detect, LOW_WiFi2 on RE2

const uint8_t Synth1_PIN_SS = 6 ;           // LO1 Slave Select or LOW_WARN on RE2
const uint8_t LOW_WARN = 6 ;                // LO1 Slave Select or LOW_WARN on RE2

const uint8_t Synth2_PIN_SS = 7 ;           // Synth2/3 Slave Select
 
const uint8_t ADF4001_PIN_CE = 10;          //  adf4001 11 Chip Enable CE onIoT33 D10 drives CP output to tri-state
const uint8_t ADF4001_PIN_MUX = 4;          //  adf4001 15 MUX on IoT33 D4 which is normally set to become PLL lock detect
const uint8_t ADF4001_PIN_SS = 5;           //  Pin 14 on adf4001 LE  SPI SS, on IoT33  D5
const uint8_t ADF4001_PIN_MOSI = 11;        //  Pin 13 on adf4001 SPI MOSI, on IoT33  D11
const uint8_t ADF4001_SCK_PIN = 13;         //  Pin 12 on adf4001 SPI SCK, on IoT33  D13
 
const String DEFAULT_SSID = "changeToYourSSID";      // initial network SSID (name),
const String DEFAULT_PASS = "changeToYourPassword";  // initial network password (use for WPA, or use as key for WEP)
//const String DEFAULT_SSID = "Corridor2";      // initial network SSID (name),
//const String DEFAULT_PASS = "1234567890";    // initial network password (use for WPA, or use as key for WEP)

const uint32_t DEFAULT_CLOCK0 = UBLOX_MAGIC_FREQ;
const uint32_t DEFAULT_CLOCK1 = freq25M;             
const uint32_t DEFAULT_CLOCK2 = freq27M;             
const uint32_t DEFAULT_CLOCK3 = freq41M;             
const uint32_t DEFAULT_CLOCK4 = freq66M;             
const uint32_t DEFAULT_CLOCK5 = freq10M;     
const uint32_t DEFAULT_CLOCK6 = freq125M;     
const uint32_t DEFAULT_CLOCK7 = freq25M; 

const uint32_t DEFAULT_POWER0 = 0b10;
const uint32_t DEFAULT_POWER1 = 0b10;
const uint32_t DEFAULT_POWER2 = 0b10;
const uint32_t DEFAULT_POWER3 = 0b10;
const uint32_t DEFAULT_POWER4 = 0b10;
const uint32_t DEFAULT_POWER5 = 0b10;
const uint32_t DEFAULT_POWER6 = 0b10;
const uint32_t DEFAULT_POWER7 = 0b10;

const disciplineType DEFAULT_DISCIPLINE = Ext10;
const boolean DEFAULT_AUTOPROMOTE = false;

const boolean DEFAULT_CPIDISABLE = false;
const uint8_t DEFAULT_CPI = 3;                       //  cp current at startup, for non-locked disciplines so doesn't matter !
const uint8_t DEFAULT_GNSSCPI = 1;                   //  cp current default for GNSS, 0 may never report locked
const uint8_t DEFAULT_EXT10CPI = 6;                  //  cp current default for external 10 MHz
const uint8_t DEFAULT_PLLMUX = 1;

const uint16_t DEFAULT_GNSSSCALING = 256;             //  pfd scaling default for GNSS
const uint16_t DEFAULT_EXT10SCALING = 64;            //  pfd scaling default for Ext10 MHz
const uint16_t DEFAULT_PFDSCALING = 128;              //  pfdscaling at startup,for non-locked disciplines so doesn't matter !

const float DEFAULT_TNOMINAL = 37;                   //  Y axis crossing for TC cal
const float DEFAULT_XTALTC = 0.0;                    //  tuning slope bits/degreeC at TNOMINAL

const boolean DEFAULT_FIRSTRUN = true;

const boolean DEFAULT_RXCONVERTERENABLED = true;     //  downcovert/receive enable
const boolean DEFAULT_RXPREAMP = false;              //  
const boolean DEFAULT_TXCONVERTERENABLED = true;     //  transmit enable
const boolean DEFAULT_TRANSCEIVE = false;            //  HF radio needs a single connection when true
const boolean DEFAULT_RX_EN = true;                  //  HF Output is from rx converter notHY Ant

const boolean DEFAULT_GNSSRESET = false;             //  allow gnssReset 1st run, starts sat acquisition from factory default
 
const uint16_t port = 8078 ;                         //  port # for web serverlity from China, WATCH OUT !  A00x60 conflicts with Si5351 !
 
#endif
