#ifndef ADF4001_h
#define ADF4001_h

#include <Arduino.h>
#include <stdint.h>
#include <SPI.h>              // needed for SPI to the 4001
#include "AAconfig.h"
#include "Reg.h"              // generic Register class for both 24 (adf4001) and 32 (adf4351) bit registers

extern struct state  activestate ;
extern boolean adf4001RegConfig;    // update this flag based on presence ofvalid SW settings
extern boolean adf4001Present;    // update this flag based on presence of HW
extern boolean adf4001Locked;

//extern uint32_t activestate.fclock0;

//  ADF4001 datasheet stuff that doesn't change with wiring, pin connections are in AAconfig.h, init settings in setup()
const uint32_t ADF4001_FREQ_MAX = 165000000UL;     //  Maximum RF PIN Frequency = value of MAX Unsigned Long  ADF4001 @ 3V
const uint32_t ADF4001_FREQ_MIN = freq10k;         //  Minimum RF PIN Frequency ADF4001
const uint32_t ADF4001_PFD_MAX  = 55000000UL;      //  Maximum PFD Operating Frequency for ADF4001
const uint32_t      ADF4001_PFD_MIN  = 300UL;           //  Minimum PFD Operating Frequency for ADF4001
const uint32_t ADF4001_REFIN_MAX = 104000000UL;    //  Maximum Reference PIN Frequency ADF4001
const uint32_t ADF4001_REFIN_MIN = freq10k;        //  Minimum Reference PIN Frequency ADF4001 unless DC coupled
const uint32_t ADF4001_R_MAX = 16383UL;            //  Highest NDivisor for ADF4001
const uint32_t ADF4001_R_Min = 1UL;                //  Lowest NDivisor for ADF4001
const uint32_t ADF4001_N_MAX = 8191UL;             //  Highest RDivisor for ADF4001
const uint32_t ADF4001_N_Min = 1UL;                //  Lowest RDivisor for ADF4001

class ADF4001
{
  public:
    ADF4001(byte ADF4001sspin, byte ADF4001cepin, byte ADF4001ldpin, uint8_t ADF4001mode, uint32_t  speed, BitOrder ADF4001order ) ;
    void ADF4001setup();                // for complete initialization, copies autosave into local values
    void ADF4001setHWRegisters() ;      // program HW registers from local values
    void ADF4001counterResetMethod();
    bool checkADF4001locked();
    Reg ADF4001_R[4] ;                  // adf4001  has 4 registers #0-3,
    boolean cpdisable;                  //  0 means current enabled
    uint8_t cpcurrent;
    uint8_t pllMux;
    uint16_t RCounter;
    uint16_t NCounter;
    uint32_t rfPinFreq ;           //  from Timepulse or Ext10,  keeping pin frequencies with the object for reporting
    uint32_t referencePinFreq ;    //  wired to Si5351 ck0
    //
  private:
    bool ADF4001checkForHW()  ;
    void ADF4001reg0ToText()  ;
    void ADF4001reg1ToText()  ;
    void ADF4001reg2ToText()  ;
    void ADF4001reg3ToText()  ;
    SPISettings  ADF4001spi_settings;

    void ADF4001writeDev( Reg r) ;    // write an entire register
};
//
#endif
