
#include "MCP23008.h"
#include "AAconfig.h"
//
Adafruit_MCP23008 mcpIO;
//
void mcpIOsetup() {
  mcpIO.begin();      // use default address 0, I2C communication
  mcpIO.pinMode(TX_EN, OUTPUT);
  mcpIO.pinMode(CONVERTER_BYPASSED, OUTPUT);
  mcpIO.pinMode(CONVERTER_ACTIVE, OUTPUT);
  mcpIO.pinMode(PREAMPA, OUTPUT);
  mcpIO.pinMode(PREAMPB, OUTPUT);
  mcpIO.pinMode(TRANSCEIVE, OUTPUT);
  mcpIO.pinMode(NOTTRANSCEIVE, OUTPUT);
}
//
//
void obeyPTT() {

  if ((digitalRead(PTT_PIN) | forceTx ) & activestate.txconverter)  {        // If PTT & Transmit is enabled or testing
    mcpIO.digitalWrite(TRANSCEIVE, false);                      // tx must connect to its own SMA, while transmitting
    mcpIO.digitalWrite(NOTTRANSCEIVE, true);
    mcpIO.digitalWrite(TX_EN, true);                            // power up the tx
  }
  else
  {
    mcpIO.digitalWrite(TRANSCEIVE, activestate.transceive);     // rx IF routed appropriately, when not transmitting
    mcpIO.digitalWrite(NOTTRANSCEIVE, !activestate.transceive);
    mcpIO.digitalWrite(TX_EN, false);                           // power down the tx
  }
}
//
//
void setPreamp(boolean preamp_state) {                  // preamp SPDT & Power (requres high-enable Power Switch)
  mcpIO.digitalWrite(PREAMPA, preamp_state);            // preamp switch is engaged when PREAMP is TRUE
  mcpIO.digitalWrite(PREAMPB, !preamp_state);
  activestate.preamp = preamp_state;
}
//
void setRxConverter(boolean rxconvert) {                  // Bypass to HF Ant select
  if (rxconvert) {
    mcpIO.digitalWrite(CONVERTER_BYPASSED, false);        // rxconverter is engaged when bypassed is false
    mcpIO.digitalWrite(CONVERTER_ACTIVE, true);
    mcpIO.digitalWrite(TRANSCEIVE, activestate.transceive);
    mcpIO.digitalWrite(NOTTRANSCEIVE, !activestate.transceive);
  }

  else {
    mcpIO.digitalWrite(CONVERTER_BYPASSED, true);
    mcpIO.digitalWrite(CONVERTER_ACTIVE, false);
    mcpIO.digitalWrite(TRANSCEIVE, true);               // HF antenna is always hooked to 20m_Out when bypassed
    mcpIO.digitalWrite(NOTTRANSCEIVE, false);           //  but must bedisconnected from tx attenuator when not converting
  }
  activestate.rxconverter = rxconvert;

  Serial.print("RX path is to ");
  rxconvert ? Serial.println("Converter") : Serial.println("HF Ant") ;
}

void setTransceive(boolean transceive_state) {                  // transceive SPDT
  mcpIO.digitalWrite(TRANSCEIVE, transceive_state);             // rx output goes to tx SMA when TRANSCEIVE is true
  mcpIO.digitalWrite(TRANSCEIVE, !transceive_state);            // rx output goes to rx SMA when TRANSCEIVE is false
  activestate.transceive = transceive_state;
  Serial.print("Transceive is ");
  activestate.transceive ? Serial.println("On") :  Serial.println("Off");
}
//
void setTxConverter(boolean txconverter_state) {
  activestate.txconverter = txconverter_state;
  Serial.print("TX Converter & TXIF are ");
  activestate.txconverter ? Serial.println("On") :  Serial.println("Off");
}
//
