/*
 * This version is from 19 May after  weeks of struggle. Many of the problems are perhaps due to testing on 5351 v C,
 * external clock input and IoT33 -  the use of SAMD rather than AVR libraries. The Si5351_read is NOT WORKING reliably. 
 * si5351c.si5351_write( 0x0011, 0x0F ) followed by si5351c.si5351_read( 0x0011 )
 * doesn't show the 0xFF that was written for an IoT33 while it DOES work for a UNO. PulseView comparisons are darn similar. Oh boy.
 *  
 * Uses NT7S library (modified_
 * Legacy support for Si5351 A parts 
 * It's assumed that only canned VCXO's are used so crystal C doesn't matter
 * Continuos fractions have been implemented. 
 * 
 * Make compilation changes for PCB type in AAconfig.h
 * [note: once can safely ignore the "warning: changing start of section .bss by 4 bytes" at end of compilation]
 */ 
#include <Arduino.h>
#include <SPI.h>
#include <Wire.h>
#include <WiFiNINA.h>
#include <stdint.h>

//  common pieces
#include "AAconfig.h"
#include "ABTypes.h"

//  necessary Reference pieces 
#include "RefGen.h"     // Si5351 and ADF4351 Reference  PLL at 0x60
#include "GNSSTime.h"   // Max-M8 GNSS rx at 0x42 or SPI

//  Additional Frequency Converter pieces
#include "MCP23008.h"   // I/0 Expander

//  Watchdog Timer
#include <Adafruit_SleepyDog.h>

//#include "ISRTimer.h"
//#include "SAMDTimerInterrupt.h"           //  include *before* FlashStorage_SAMD to Make sure BOARD_NAME isn't set to Unknown
//SAMDTimer ITimer(SELECTED_TIMER);
 
//  EEPROM emulation for storing start-up state
#include <FlashStorage_SAMD.h>
 
//  Local and extern declarations
UBX_NAV_SAT_data_t *ubxDataStruct ;
uint8_t numSVs;
disciplineType  discipline, prevdiscipline = XO;          //  need to know about these for reporting
uint32_t Ftune; //  Desired center Frequency in MHz       // = LO2FREQ / freq1M - activestate.lo1Freq / freq1M;
 
extern int main();
extern void changeDiscipline(disciplineType  discipline);
 
extern Si5351 si5351c;
extern boolean si5351cSetup();

extern boolean WebServersetup();
extern void WebServerloop();
 
extern void obeyPTT();
 
extern IPAddress  ip, rip;
 
//   flags for HW inventory
boolean wiFiModulePresent = false;        // updated in WebServer
boolean tpCapableGNSSPresent = false;     // updated by GNSSsetup
boolean adf4001Present = false;           // updated by ADF4001.checkForHW()
boolean si5351Present = true;             // updated in si5351cSetup()
boolean DACPresent = false;               // updated by DAC47x6
 
//  flags for HW functions
boolean adf4001Locked  = false;
boolean satLocked;
boolean timePulsePresent = false;
boolean forceTx  = false;                 //  Override PTT, force continuous transmit for testing
 
boolean activeStateHasChangedfromWeb = false;   //  used to update terminal if desired
boolean watchdogTimerEnabled = false;     //  
 
struct state defaultstate, activestate ;
 
String inString;                                // User input String
char inchar[11];                                // User input char array
 
float PCBTemperature ;
float calcArefV = 3.300;                        //  calculated from measurement  (seeded Volts)

//  instantiate HW

SFE_UBLOX_GNSS myGNSS;                          //  uBlox GNSS rx module
 
ADF4001 cleanupPLL( ADF4001_PIN_SS,  ADF4001_PIN_CE,  ADF4001_PIN_MUX,  SPI_MODE0,    2500000UL,  MSBFIRST ) ;
 
MCP47x6base * myDAC = new MCP4726(MCP47x6_defaultaddr); //  Using the MCP4726 12 bit version
 
// Synthesizers for the converters, Synth1 produces L01 while Synth2 produces LO2&LO3
ADF4351 Synth1(Synth1_PIN_SS, Synth1_PIN_CE, Synth1_PIN_LOCK, SPI_MODE0, 2500000UL, MSBFIRST) ; // (SPI)
ADF4351 Synth2(Synth2_PIN_SS, Synth2_PIN_CE, Synth2_PIN_LOCK, SPI_MODE0, 2500000UL, MSBFIRST) ; // (SPI)
 
void(* resetFunc) (void) = 0;                   // create a standard reset function for watchdog
 
void reportInventory()  {
  //  Serial.println("\nModule Inventory:");
  //update presence and function flags
  si5351Present  ?  Serial.println("si5351 HW Present") : Serial.println("\tsi5351 HW NOT Present");            // updated by RefGen
  DACPresent   ?  Serial.println("DAC HW Present") :  Serial.println("\tDAC HW NOT Present");                   // dac for the TCXO
  adf4001Present   ?   Serial.println("adf4001 HW Present") :  Serial.println("\tadf4001 HW NOT Present");      // updated by adf4001
  tpCapableGNSSPresent ?  Serial.println("tp Capable uBlox Present") : Serial.println("\ttp Capable uBlox HW NOT Present");               // updated by GNSSTime
  wiFiModulePresent ? Serial.println("wiFi HW Present") : Serial.println("\twiFi HW NOT Present");                   // updated by RefWebServer

  // Serial.println("\nFunction Inventory:");
  if (DACPresent)Serial.println("DAC available"); else Serial.println("\tDAC NOT available");
  if (timePulsePresent) Serial.println("timePulse Present"); else Serial.println("\ttimePulse NOT yet Present");
  if (cleanupPLL.checkADF4001locked()) Serial.println("cleanupPLL Locked"); else Serial.println("\tcleanupPLL not yet Locked");

  Serial.print("PCB Temp is "); Serial.print( readPCBTemp(), 1  ); Serial.println("C");
  //
  if (!COMPILED_FOR_REFERENCE) {
    if (activestate.rxconverter) Serial.println("RX Converter Enabled"); else Serial.println("HF Ant Enabled");
    if (activestate.preamp) Serial.println("Preamp Enabled"); else Serial.println("Preamp Disabled");
    if (TRANSVERTER)  {
      if (activestate.txconverter) Serial.println("TX Converter Enabled"); else Serial.println("TX Disabled");
    }
  }
} //  end reportInventory
 
String getDisciplineName( disciplineType discipline) {  //  return English of discipline for reporting
  switch (discipline) {
    case Search:
      return ("Search");
      break;
    case XO:
      return ("XO");
      break;

    case TCXO:
      return ("TCXO");
      break;

    case GNSS:
      return ("GNSS");
      break;

    case Ext10:
      return ("Ext10");
      break;

    default:
      return ("None");
      break;
  }
}
 
void printLOpowers()  {                               //  print individual adf431 output stage settings FOR TESTING
  //  We don't get B output power unless A output is enabled !
  char pwr;
  //Synth1.synthPowerUp();            //    Enables Synth1A+ output power, B output controlled after this is ON
  Serial.print("Synth1A+(rxLO1) = "); pwr = Synth1.R[4].getbf(5, 1 ); Serial.print(pwr, BIN) ;
  Serial.print("\tSynth1B++(txLO1) = "); pwr == 1 ? Serial.println("on") : Serial.println("off");
  if (!TRANSVERTER)  {    // turn off Synth1B+ power if FE
    Synth1.R[4].setbf(8, 1, 0) ;
    Synth1.ADF4351writeReg(  Synth1.R[4]) ;
    Serial.println("Synth1B++ unused for FE "); ;
  }
  //
  //  Synth2 A+(rxLO3), A-(txLO3), B+(rxLO2), B-(txLO2)
  //Synth2.synthPowerUp() ;           // We don't get B output power unless A output is enabled !
  Serial.print("LO3A+(rx) LO3A-(tx) = "); pwr = Synth2.R[4].getbf(5, 1 );      Serial.println(pwr, BIN) ;
  Synth2.R[4].setbf(8, 1, 1) ;      // Turn on Synth2 B (2.352 GHZ RF power)
  //  Synth2.ADF4351writeReg(  Synth1.R[4]) ;
  Serial.print("LO2B+(rx) LO2B-(tx) = "); pwr = Synth2.R[4].getbf(8, 1 ); pwr == 1 ? Serial.println("on") : Serial.println("off");
}   //  end printLOpowers
 
void SetSynth1(uint32_t lo1Freq) {                    // called by the Web Server for tuning
  if ( Synth1.ADF4351setf(lo1Freq) == 0 )   {  }
  else {
    Serial.println("ino: LO1 setf() error") ;
  }
}
 
void PrintValues(ADF4351 LO)  {
  Serial.print(" set to ( ");
  Serial.print(LO.ADF4351N_Int);
  if (!(LO.ADF4351Frac == 0))  {
    Serial.print("+(");
    Serial.print(LO.ADF4351Frac);
    Serial.print("/");
    Serial.print(LO.ADF4351Mod);
    Serial.print(")");
  }
  Serial.print(" ) * ");

  Serial.print(LO.ADF4351PFDFreq / 1e6, 6);
  if (!(LO.ADF4351outdiv == 1))  {
    Serial.print("/");
    Serial.print(LO.ADF4351outdiv);
  }
  Serial.print(" = ");
  Serial.print(LO.ADF4351cfreq / 1e6, 6) ;
  Serial.println(" MHz");

  //Synth1.ADF4351reg4ToText();
}   //  end PrintValues
 
void setupLOs() {                                     //  for Converters
  //Serial.println("\nSetting up LOs");
  //
  /*!   enable innards and output currents before programming CE enable (ADF4351pinCE, HIGH) the entire 4351
  */
  Synth1.ADF4351init() ;       // initializes IoT SPI pin I/O directions & CE enable
  Synth2.ADF4351init() ;       //
  Synth1.synthPowerUp() ;      // get both adf4351's ready to program, Chip Power on for both 4351s
  Synth2.synthPowerUp() ;      //
  Synth1.auxOutEnable(true);   //  enable Synth1B+ so output can go below 2200 MHz
  Synth2.auxOutEnable(true);   //  Synth2 B enabled
  /*!     setup the chip SW object (for a 20 MHz ref freq)    most of these are defaults
  */
  Synth1.ADF4351MainPwrlevel = 3 ;        //  0:3 sets LO1 A mixer power -5 to +4 dBm output
  Synth1.SynthBPwrlevel = 3 ;             //  sets LO1 B power (for tx)
  Synth1.ADF4351Refdoubler = 0 ;          //  ref doubler off
  Synth1.ADF4351Refdivider = 0 ;          //  ref divider off
  Synth1.ADF4351TimeoutCounter = 150 ;
  Synth1.ADF4351BandSelClock = 160 ;      //  20M/160=125 kHz (the maximum)
  Synth1.ADF4351RCounter = 1 ;            //  R counter to 1 (no 10 bit division)
  Synth1.ADF4351ChanStep = freq1M ;

  Synth2.ADF4351MainPwrlevel = 3 ;        //  sets LO3 mixer power
  Synth2.SynthBPwrlevel = 2 ;             //  sets Synth2 mixer power
  Synth2.ADF4351Refdoubler = 0 ;          //  ref doubler off
  Synth2.ADF4351Refdivider = 0 ;          //  ref divider off
  Synth2.ADF4351TimeoutCounter = 150 ;
  Synth2.ADF4351BandSelClock = 160 ;
  Synth2.ADF4351RCounter = 1 ;            //  R counter to 1 (no division)
  Synth2.ADF4351ChanStep = freq1M ;
  //
  if (  !(Synth1.ADF4351setref(UWVREFERENCE) ==  0) ) {   //  validate the common uwvreference frequency
    Serial.println("Synth1 & Synth2 ref freq set error") ;
  }
  //
  //  the setf()'s are where the call to hardware write happens, up to now it was only the SW object
  if ( Synth1.ADF4351setf(activestate.lo1Freq) == 0 ) { //  validate, set & print the Synth1 frequency
    Ftune = activestate.lo1Freq / freq1M - 2185;        //  initial setting for Ftune
    //    Serial.print("LO1 core  = ");
    //    Serial.print(Synth1.ADF4351outdiv * (activestate.lo1Freq / 1e6), 6);
    //    Serial.println(" MHz");
    //
    Serial.print("LO1 output ") ;
    PrintValues(Synth1);
  } else {
    Serial.println("Synth1.setf error") ;
  }
  //
  if ( Synth2.ADF4351setf(LO3FREQ) == 0 ) {  // Setting LO3 the divided, 147 MHz Primary/A output frequency of 2nd synthesizer, LO2FREQ is undivided/uwave on Aux/B
    //
    Serial.print("Synth2B output (Synth2.ADF4351outdiv * LO3FREQ) = ");
    Serial.print(Synth2.ADF4351outdiv * (LO3FREQ / 1e6), 6);
    Serial.println(" MHz");

    Serial.print("LO3 output ") ;
    PrintValues(Synth2);
  } else {
    Serial.println("Synth2.setf error") ;
  }
} //  end setupLOs
 
void printState(state &state)  {                      //  print a state object
  Serial.print("currently web page is ");   // reminder of where to go with a browser:
  if(ip != "0.0.0.0") {
  Serial.print("http://");
  Serial.print(ip);
  Serial.print(":");
  Serial.println(port);
  }
  else 
  Serial.println("unavailable");
  //
  Serial.println("=============================================================");
  Serial.print("01) *SSID = "); Serial.println(state.ssid);
  Serial.print("02) *PASSWORD = "); Serial.println(state.pass);
  if (COMPILED_FOR_REFERENCE) {
    Serial.print("03) clock1 = ");
    Serial.println(state.fclock1);
  }
  Serial.print("04) user clock2 = "); Serial.println(state.fclock2);
  Serial.print("05) discipline = "); Serial.println (getDisciplineName(state.discipline) );
  Serial.print("06) autoPromote = "); state.autoPromote ? Serial.println("true") : Serial.println("false");
  Serial.print("07) cpcurrent = "); Serial.println(state.cpcurrent, 1);
  Serial.print("08) *CPI DISABLE = "); state.cpdisable ? Serial.println("true") : Serial.println("false");

  Serial.print("09) pfdScaling = "); Serial.println(cleanupPLL.RCounter);
  Serial.print("10) *XO DAC = "); Serial.println(state.refVCXOdac);
  Serial.print("11) *XO TEMP = "); Serial.print(state.refVCXOT, 1); Serial.println("C");
  Serial.print("12) *XO TC = "); Serial.println(state.vcxoTC);
  //  Serial.print("13) cload = "); {
  //    if (state.cload == Si5351cRYSTAL_LOAD_6PF) Serial.println("6 pF");
  //    else if (state.cload == Si5351cRYSTAL_LOAD_8PF) Serial.println("8 pF");
  //    else if (state.cload == Si5351cRYSTAL_LOAD_10PF) Serial.println("10 pF");
  //  }
  if (!COMPILED_FOR_REFERENCE) {
    Serial.print("14) lo1Freq = ");
    Serial.println(state.lo1Freq);
    if (TRANSVERTER)  {
      if (TRANSCEIVE)Serial.print("15) transceive = "); state.transceive ? Serial.println("true") : Serial.println("false");
      Serial.print("16) txconverter = "); state.txconverter ? Serial.println("true") : Serial.println("false");
    }
    Serial.print("17) rxconverter = "); state.rxconverter ? Serial.println("true") : Serial.println("false");
  }
  Serial.println("(UPPER case *parameters can be modified here, others from web)");
  Serial.println("=============================================================\n");
}
 
void writeStateToEEPROM(state &stateToWrite) {        //  writes active state object to EEPROM DON'T OVERUSE THIS or HW will wear out**NOFLASH**
  int eeAddress = START_ADDRESS + sizeof(WRITTEN_SIGNATURE);   //Location we want the data to be put.
  //One simple call, with the address first and the object second.
  EEPROM.put(eeAddress, stateToWrite);

  if (!EEPROM.getCommitASAP()) {
    Serial.println("CommitASAP not set. Need commit()");
    EEPROM.commit();
  }
}
 
float getValidFloat(String prompt) {                  //  used for modifying active state from serial terminal
  String inString = promptInput("", false);           // user response String//  CAUTION THIS IS A BLOCKING METHOD !!!!
  String outString = "";
  uint8_t inChar;
  //
  for (int idx = 0; idx < sizeof(inString); idx++)  { //  walk through the user's response
    int inChar = inString[idx];
    if (isDigit(inChar) ) { //
      // convert the incoming byte to a char and add it to the string:
      outString += (char)inChar;
    }
  }
  return (outString.toFloat());
}
 
uint16_t getValidUint16(String prompt) {              //  used for modifying active state from serial terminal
  String inString = promptInput("", false);           // user response String//  CAUTION THIS IS A BLOCKING METHOD !!!!
  String outString = "";
  uint8_t inChar;
  //
  for (int idx = 0; idx < sizeof(inString); idx++)  { //  walk through the user's response
    int inChar = inString[idx];
    if (isDigit(inChar) ) { //
      // convert the incoming byte to a char and add it to the string:
      outString += (char)inChar;
    }
  }
  return (outString.toInt());
}
 
void EEPROMtoActivestate() {                          //  reads EEPROM into active state object **NOFLASH**
  int eeAddress = START_ADDRESS + sizeof(WRITTEN_SIGNATURE);   //Location we want the data to be put.
  //One simple get, with the address first and the object second.
  EEPROM.get(eeAddress, activestate);

  if (!EEPROM.getCommitASAP()) {
    Serial.println("CommitASAP not set. Need commit()");
    EEPROM.commit();
  }
}
 
void modifyActiveState(String Mstring  )  {           // used when input was "Mxx"for modifying active state from serial terminal
  //
  String parameterString = "  ";
  if (isDigit( Mstring[1] )  )     parameterString.setCharAt(0, Mstring[1]); else {};
  if (isDigit( Mstring[2] )  )     parameterString.setCharAt(1, Mstring[2]);  else {};

  int switchval = parameterString.toInt(); //toInt() Converts a valid String to an integer.
  // Mstring[1] should be an integer number.
  // If the String contains non-integer numbers, the function will stop performing the conversion.
  //  Serial.print("int of two char parameter argument is ")
  while (Serial.available() > 0) {                          //  flush input buffer
    char  t = Serial.read();
  }
  //
  String promptresponseString;
  switch (switchval) {
    case 1:   //  ssid is a String
      Serial.print("\nEnter New SSID\n");
      promptresponseString = promptInput("\nNew SSID is ", false);//   CAUTION THIS IS A BLOCKING METHOD !!!!
      promptresponseString.toCharArray(activestate.ssid, 32);     //  convert to char [] and save to state
      Serial.println(activestate.ssid);
      break;
    case 2:  //  'pass' is a String
      Serial.print("\nEnter New PASSWORD\n");
      promptresponseString = promptInput(" New PASSWORD is ", false);//     CAUTION THIS IS A BLOCKING METHOD !!!!
      promptresponseString.toCharArray(activestate.pass, 32); //  convert to char [] and save to state
      Serial.println(activestate.pass);
      break;
    //    case 3 :  // clock1 is a uint32_t
    //   String str = promptInput(" New Clock1 is ");//     CAUTION THIS IS A BLOCKING METHOD !!!!
    //   str.toCharArray(chararray, 11);
    //      activestate.fclock =    atoi(chararray)  ;
    //      Serial.println(activestate.fclock);
    //      break;
    case 8:
      Serial.print("\nCharge Pump disable?\n");
      activestate.cpdisable = (boolean) getValidUint16("\nEnter new charge pump value, 0 is cp ON, 1 = cp OFF\n");
      Serial.print("New cpdisable is ");
      Serial.println(activestate.cpdisable);
      break;
    case 10:
      Serial.print("\nEnter NEW XO DACsetting\n");
      activestate.refVCXOdac = getValidUint16("\nEnter new XO reference dacsetting\n");
      Serial.print("New reference dacsetting is ");
      Serial.println(activestate.refVCXOdac);
      setHWDiscipline(  activestate.discipline);  //  update from new setting
      break;
    case 11:
      Serial.print("\nEnter NEW XO T\n");
      activestate.refVCXOT =  getValidFloat("\nEnter new XO reference Temperature\n");
      Serial.print("New reference temperature is ");
      Serial.println(activestate.refVCXOT);
      setHWDiscipline(  activestate.discipline);  //  update from new setting
      break;
    case 12:
      Serial.print("\nEnter NEW  XO TC\n");
      activestate.refVCXOT = getValidFloat("\nEnter new XO reference TC\n");
      Serial.print("New reference TC is ");
      Serial.println(activestate.vcxoTC);
      setHWDiscipline(  activestate.discipline);  //  update from new setting
      break;
    default:
      Serial.print("No Modification for M");
      Serial.print(switchval, DEC);
      Serial.println(" is enabled yet");
      break;
  }
}
 
String promptInput(String prompt, boolean upper)  {   //  return a  string response to a prompt, or "X" if no response, capitalized if boolean is set
  //    CAUTION THIS IS A BLOCKING METHOD !!!!
  Serial.print(prompt);
  const int waittime = 60000;
  int beginwait = millis();
  int endwait = millis() + waittime;
  while (1)  {                    //    CAUTION THIS IS A BLOCKING METHOD !!!!
    Watchdog.reset();             //  cautionary in case waittime set to exceed watchdog timer
    if (millis() > endwait ) {   // but don't wait forever
      inString = "X";
      Serial.println("no user response, going back to work.");
      break;
    }
    //
    if (Serial.available()) {
      inString = Serial.readString(); // The function does not terminate early if the data contains end of line characters.
      //  The returned String may contain carriage return and/or line feed characters if they were received.
      if (upper)     inString.toUpperCase();
      inString.trim();   // remove  whitespace in String
      break;
    }
  }
  return (inString);
} //  end promptInput
 
/*  TimerHandler()  in case I need it later
  //volatile bool toggle = false;                     //  initialized to false but gets read every time
  //volatile uint32_t count = 0;
  //volatile uint8_t regval;
  //err_t error;
  //
  //void TimerHandler() { //  (ISR) currently trying to flip phase of CLK2 output to see how quickly it can be done
  //  //timer interrupt toggles pin TEST_PIN
  //  digitalWrite(TEST_PIN, toggle);
  //  toggle ?  clockgen.myWrite8(SI5351_REGISTER_18_CLK2_CONTROL, regval && 0xEF)    :   clockgen.myWrite8(SI5351_REGISTER_18_CLK2_CONTROL, regval || 0x10) ;
  //  toggle = !toggle;
  //}
  //
*/
 
float calculateVref(uint16_t dacset) {
  /* Use DAC4V @dacset to calculate ADC Aref equivalent, returns calculated analog ADC reference V.

          this is necessary because of ambiguous IoT33 documentation and protection resistor(recommended?) between actual 4V
          regulator and  IoT33 p18. PRotection is present because of a warning that too much current injected into that pin when
          something other than AR_EXTERNALis selected as ADC reference might destroy the part.  I'm unclear if this is still a
          concern but not knowing for sure a resistor is present.
          Current best? documentation says the ADC reference params are:
                                                              Calculated Aref when 4.7k is present
          AR_DEFAULT: the default analog reference of 3.3V      3.423
          AR_INTERNAL: a built-in 2.23V reference               2.275
          AR_INTERNAL1V0: a built-in 1.0V reference             2.979
          AR_INTERNAL1V65: a built-in 1.65V reference           1.726
          AR_INTERNAL2V23: a built-in 2.23V reference           2.275
          AR_EXTERNAL: the voltage applied to the AREF pin      3.265/3.276/
  */
  float Vref;
  float dacV = (  AREF) * (dacset / 4096.0) ;  //  Output of 12 bit DAC, mV, given AREF(~4V) DAC reference
  writeDAC(dacset);                //  set DAC to ~2V
  analogReadResolution(12);
  analogReference(AR_EXTERNAL);
  //  analogReference(AR_INTERNAL2V23);
  delay(5);
  Serial.print("Measured (dacv on p21) (");
  uint16_t toss = analogRead(VSET_PIN);             //  throw away a zeroeth read to engage the reference
  delay(5);                                         //  wait for V to settle
  uint16_t db = averageBitRead(VSET_PIN);
  Vref = dacV * (4096.0 / (float) db) ;             //  Volts
  Serial.print ( db   );
  Serial.print(" bits) ==>  ");
  Serial.print(" Analog Reference = ");
  Serial.print ((float)   Vref, 3 );                //  calculate effective ADC reference
  Serial.println(" VDC");
  return Vref;
}
 
void setup()  {   // this is 'main' .ino setup, called at startup and followed by a call to loop()
  Serial.end();
  Serial.begin(57600) ;       //  sets up Serial communications to host computer.
  uint16_t tstart = millis();
  while (!Serial && ((millis() -tstart ) < 2000 ) ) { // wait for serial port to connect. (Needed for native USB ports such as IoT33 only)
    delay(100);             // for host/IoT33 path to settle  On Linux host first stuff doesn't print without a ~1000 msec wait.
  }
  //
  pinMode(LOW_WARN, OUTPUT) ;               // preset LEDs
  pinMode(LOW_WiFi1, OUTPUT) ;
  pinMode(LOW_WiFi2, OUTPUT) ;
  digitalWrite(LOW_WARN, HIGH);
  digitalWrite(LOW_WiFi1, HIGH);
  digitalWrite(LOW_WiFi2, HIGH);
  
  watchdogTimerEnabled = false;           //  put here for timeout/restart case. Compiler already set it false.
  const int waittime = 7000;              //  Don't exceed watchdog timer here (if it's running) !
  //  Wait for host computer USB to become available so startup messages can be seen by user since connection may have been broken
  Serial.println("*********************************************************");
  Serial.println("* Waiting for USB connection, <CR> if ready to continue *");
  Serial.println("*********************************************************");
  int endwait = millis() + waittime;
  while (true)  {                         //  wait or break out early if user has USB access and wants to bypass the wait
    if (millis() > endwait ) {            // but don't wait forever
      break;
    }
    //
    if (Serial.available()) {                 //  user USB answered so continue without further ado
      //Serial.flush();                         //  clears transmit buffer
      String inString = Serial.readString();  // flush the incoming buffer
      break;
    }
  }
 
  { //  Print FW version
    Serial.println("-------------------------------") ;
    Serial.print (F(__DATE__ " " __TIME__));
    Serial.print(" v ") ;
    Serial.println(SWVERSION) ;
  }
 
  BigNumber::begin ();                    // initialize BigNumber library
  //
  if (COMPILED_FOR_REFERENCE)  Serial.println("Compiled as Reference") ;
  else (TRANSVERTER) ? Serial.println("Compiled for Transverter") : Serial.println("Compiled for Frequency Extender") ;
 
  { /*  First thing is to take care of setting up default state for firstrun 
      //  First time after upload activestate will come from CONST defaultstate settings defined at end of AAconfig.h,
      power-on and resets after that  will use activestate stored in eeprom which may have been modified and resaved
    */
    DEFAULT_SSID.toCharArray(defaultstate.ssid, 32);   //  convert to char [] and save to fixed size state
    DEFAULT_PASS.toCharArray(defaultstate.pass, 32);   //  convert to char [] and save to fixed size state
    defaultstate.fclock0 = DEFAULT_CLOCK0;
    defaultstate.fclock1 = DEFAULT_CLOCK1;
    defaultstate.fclock2 = DEFAULT_CLOCK2;
    defaultstate.fclock3 = DEFAULT_CLOCK3;
    defaultstate.fclock4 = DEFAULT_CLOCK4;
    defaultstate.fclock5 = DEFAULT_CLOCK5;
    defaultstate.fclock6 = DEFAULT_CLOCK6;
    defaultstate.fclock7 = DEFAULT_CLOCK7;

    defaultstate.pwr[0] = DEFAULT_POWER0;

    defaultstate.pwr[1] = DEFAULT_POWER1;
    defaultstate.pwr[2] = DEFAULT_POWER2;
    defaultstate.pwr[3] = DEFAULT_POWER3;
    defaultstate.pwr[4] = DEFAULT_POWER4;
    defaultstate.pwr[5] = DEFAULT_POWER5;
    defaultstate.pwr[6] = DEFAULT_POWER6;
    defaultstate.pwr[7] = DEFAULT_POWER7;




    defaultstate.discipline = DEFAULT_DISCIPLINE;
    defaultstate.autoPromote = DEFAULT_AUTOPROMOTE;

    defaultstate.cpdisable = DEFAULT_CPIDISABLE;

    defaultstate.GNSSpfdScaling = DEFAULT_GNSSSCALING;
    defaultstate.EXTpfdScaling = DEFAULT_EXT10SCALING;
    defaultstate.pllMux = DEFAULT_PLLMUX;
    //
    defaultstate.dacsetting = DEFAULT_DACSETTING;   //At first run dacestting and default parameters get used, modified later
    defaultstate.refVCXOdac = DEFAULT_DACSETTING;
    defaultstate.refVCXOT   = DEFAULT_TNOMINAL;
    defaultstate.vcxoTC     = DEFAULT_XTALTC;
    //
    defaultstate.firstrun = DEFAULT_FIRSTRUN;                      //  probably always true.
    //
    defaultstate.rxconverter = DEFAULT_RX_EN;                     //  single cable HF interface?
    defaultstate.preamp = DEFAULT_RXPREAMP;
    defaultstate.txconverter = DEFAULT_TXCONVERTERENABLED;        //  Transmitting enable
    defaultstate.transceive = DEFAULT_TRANSCEIVE;                 //  single cable HF interface?

    defaultstate.gnssReset = DEFAULT_GNSSRESET;                   //  reset gnss module at 1st run?
    defaultstate.lo1Freq = DEFAULT_LO1FREQ;                       //
  }
 
  { /*  Set or only retrieve previous activestate from Flash storage  */  // **NOFLASH**
    // Flash Storage; content of the FlashStorage is erased each time a new sketch is uploaded
    // Check signature at address 0 to see if this first time run or if EEPROM has already been written to with default state data
    int signature;
    int eeAddress;
    EEPROM.get(START_ADDRESS, signature);
    if (signature != WRITTEN_SIGNATURE)   {           // If the EEPROM.activestate doesn't yet store valid data, there's no WRITTEN_SIGNATURE so use defaults
      Serial.println("\nFirstrun. EEPROM doesn't yet contain valid data, writing default to power-on state");
      EEPROM.put(START_ADDRESS, WRITTEN_SIGNATURE);   //  first the signature which indicates active data present
      //
      defaultstate.firstrun = true;                   //  always start by flagging this as the first time new code has been executed
      activestate = defaultstate;                     //  seed the active state fresh with the default values
      writeStateToEEPROM(defaultstate); //  **NOFLASH**
    }                                                 // end of 1st store after an upload
    else  {                                           // EEPROM.activestate is already seeded and perhaps modified. use it.
      EEPROMtoActivestate();                          //  activestate uses whatever we have saved rather than default  **NOFLASH**
      activestate.firstrun = false;
    };
    Serial.println("\nPower-On State is:");
    printState(activestate);
  } //  end Take care of Flash storage
  /**************************************************************  Next get IoT33 on-board resources******************************************/
  analogReference(AR_EXTERNAL); // external reference as measured at Aref pin  But NOT   working always using INTERNAL/DEFAULT **TODO**
  analogReadResolution(12);     // use full IoT33 ADC resolution, (library may already default to 12 bits)
 
  Serial.print("PCB temperature is ");
  Serial.print( readPCBTemp(), 1  );
  Serial.println("C");
 
  /*****************************   Then get off-board modules common to both Reference & Converters *****************************************/
  Wire.begin() ;                            //  called one time (here) with no address passed, sets this host up as master.
  SPI.begin();                              //  Need SPI to talk to si5351,  ADF4001 and maybe  GNSS module
  
  Watchdog.disable();                       //  in case we got here by a timeout, don't want to create a loop
  si5351Present = si5351cSetup();           // (I2C) verify and set up a functioning synthesizer, set 'si5351present' flag if succeeded

  DACPresent = myDACsetup();
  if (DACPresent)  {                      //  get a better ADC refV, Volts
    calcArefV = calculateVref(2048);
    writeDAC(activestate.dacsetting);       //  in case myDACsetup didn't get close?
  }
 
  cleanupPLL.ADF4001setup();                // sets up SPI, MUX, CPI, scaling & R&N Dividers based on activestate
 
  tpCapableGNSSPresent = GNSSsetup();       // tpCapableGNSS via I2C or UART#1 ??
  if (!tpCapableGNSSPresent ) {             // Try uBlox a second time
    Serial.println("Try uBlox a second time");
    tpCapableGNSSPresent = GNSSsetup() ; // try twice if no connection first time, NINA seems to have trouble the first time, don't know why
  }
 
  changeDiscipline(activestate.discipline); // this will either explicitly set or SEARCH and set the best discipline
  
  /*****************************   Finally add HW unique to Converters  **************************************** */
  if (!COMPILED_FOR_REFERENCE) {            //  converter & preamps presets
    //
    mcpIOsetup();                           // (I2C) writes to prepare the IO Expander and preset RF switches,
    //
    setupLOs();
    //  initialize booleans.  [HFbypass, preamp, txconverter, transceive paths] & PTT
    setRxConverter(activestate.rxconverter);            //  HF Ant vs rxconverter selection
    setPreamp(activestate.preamp);
    setTxConverter(activestate.txconverter);  //  enable transmitting when PTT permits
    setTransceive(activestate.transceive);    //  routes via  rx SMA/tx SMA (single connection to HF IF radio)
    Serial.println("======== Done with HW setup, starting Web Server and watchdog ============");
    if (TRANSVERTER)  {                       //  install interrupt to control transmit via PTT when txing is enabled
      pinMode(PTT_PIN, INPUT);                //  on an IoT, only pins 2, 3, 9, 10, 11, 13, A1, A5, A7 can be used
      attachInterrupt( digitalPinToInterrupt(PTT_PIN), obeyPTT, CHANGE); // change on PTT pin calls pttHandler(ISR)
      obeyPTT();                                //  initilize transmit power
    }
  }

  boolean APconnected =  WebServersetup() ;   //  Doing this BEFORE enabling watchdog since it seems to have problems and can take a long time
   if (!APconnected)   { // Trying once more out here in main setup
   APconnected = WebServersetup() ; 
   APconnected ? Serial.println("\nFinally connected in last ditch effort\n")  
   : Serial.println ("\n****************** NOT connected to an AP. Maybe retry using \"W\"\? ********************  \n");
   } 
   
 watchdogTimerEnabled = true;             //  Enabling it now that all setup is complete
 if (watchdogTimerEnabled)
  { /* Enable the Watchdog Timer */
    int countdownMS = Watchdog.enable(8000);
    Serial.print("Enabling ");
    Serial.print(countdownMS, DEC);
    Serial.println(" ms watchdog timer.\n");
  }
 
  reportInventory();                        //  report what we have found
 
  /*  //    Leaving all this in case of future use for timed interrupts
    //    Timer Interrupt handler setup
    //    pinMode(TEST_PIN, OUTPUT);

    //  boolean BPSK = false;
    //  if ( BPSK) {  //  Timer Interrupt handler setup
    //    Serial.print(F("\nStarting Interrups on ")); Serial.println(BOARD_NAME);
    //    Serial.println(SAMD_TIMER_INTERRUPT_VERSION);
    //    Serial.print(F("CPU Frequency = ")); Serial.print(F_CPU / 1000000); Serial.println(F(" MHz"));
    //    // Interval in ms or us
    //    //if (ITimer.attachInterruptInterval_MS(TIMER_INTERVAL_MS, TimerHandler))
    //    if (ITimer.attachInterruptInterval(TIMER_INTERVAL_US, TimerHandler))
    //    {
    //      preMillisTimer = millis();
    //      Serial.print(F("Starting  ITimer OK, millis() = ")); Serial.println(preMillisTimer);
    //    }
    //    else
    //      Serial.println(F("Can't set ITimer. Select another freq. or timer"));
    //  }
    //  int i, starttime = millis();
    //  err_t error = clockgen.myRead8(SI5351_REGISTER_18_CLK2_CONTROL, &regval);
    //int count = 10000;
    //  for (i = 1; i < count; i++) {
    //    regval &= 0xEF;
    //    error = clockgen.myWrite8(SI5351_REGISTER_18_CLK2_CONTROL, regval);
    //
    //    regval |= 0x10; //  invert output phase
    //    error = clockgen.myWrite8(SI5351_REGISTER_18_CLK2_CONTROL, regval);
    //  }
    //  Serial.print("access time =  "); Serial.print((millis() - starttime)/(2.0*count) );Serial.print(" ms");
  *///

}   //  end setup
 
boolean fastBlink, slowBlink;
boolean neededHWnotPresent;
boolean lockModeButNotLocked; 
uint8_t ledCounter = 0;
uint8_t tcTableSize = 0;                            // no TC table at first, only defaults 
const uint16_t housekeepingdelay = 50;
uint32_t rechecktime = millis() + 30000;
 
void loop() {                                       // this is .ino 'Main' where things spin forever unless manual or watchdog reset.

  Watchdog.reset();

  if (wiFiModulePresent) WebServerloop();           //  Service the webserver

  if (Serial.available() > 0) {                     // Process any USB input, will block web service
    inString = Serial.readString();                 // read the incoming String:
    inString.toUpperCase( );
    inString.toCharArray(inchar, 4);
    Serial.println("\nActive State:");
    printState(activestate);

    Serial.println("======== Use 'A' to save Active State to Power-On state");
    Serial.println("======== Use 'D' to save Default State to Power-On state");
    Serial.println("======== Use 'Mxy' to modify a parameter e.g. 'M10' to modify dacsetting");
    Serial.println("======== Use 'S' or <CR> to show Active State");
    Serial.println("======== Use 'W' to (Re)Connect WiFi & Webserver");
    Serial.println("======== Use 'X' to Reset/Restart, (may terminate this USB serial connection) ");
    //
    String tempString = inString;                   //  Don't want to alter user response, LC may be required for other uses.
    tempString.toUpperCase();                       // admsx ==> ADMSX
  
    char switchchar = (char) tempString[0];
    switch (switchchar) {
      case 'A':
        writeStateToEEPROM(activestate);  //  **NOFLASH**
        Serial.println("\nActive State written to EEPROM/Power-On State");
        break;
      case 'D':
        writeStateToEEPROM(defaultstate); //  **NOFLASH**
        Serial.println("\nDefault State written to EEPROM/Power-On State, Until power-on or reset, active State remains = \n");
        break;
      case 'M':
        modifyActiveState(inString);  //   This may BLOCK THE WEB SERVER waiting on the user !!!!
        break;
      case 'S':
        break;
      case 'W':
        WebServersetup();
        break;
      case 'X':
        WiFi.end();   //  Turns off the WiFi module. If WiFi.begin() was used to connect to an access point, the connection will be disconnected.
        resetFunc();  // reset the Arduino to top of code via software function, should mimic a power cycle but not wash out GNSS ephemeris and other settings
        break;
      default:
        Serial.println("\nNo Changes");         // shouldn't ever execute
        break;
    } //  end of switch
  }   //  end of Serial.avaiable

  if (millis() >  rechecktime) {                    //  Slower Housekeeping every [30] seconds
    readPCBTemp();  //  updates PCBTemperature
    if (cleanupPLL.checkADF4001locked() & (tcTableSize == 0)  ) { // This is first lock after startup, use it to fill initial T&DAC point
      activestate.refVCXOT = PCBTemperature;                      // TC Table can build on this temperature/dacvalue point
      Serial.print(" new refVCXOT is "); Serial.print(activestate.refVCXOT, 1); Serial.println(" C");
      // DACV_PER_BIT is a constant float
      uint16_t lbits = averageBitRead(VTUNE_PIN);
      Serial.print(" ADC bits during lock "); Serial.print(lbits ); Serial.println(" bits");
      float Vtune = ((float) lbits / 4096.0) * calcArefV;
      Serial.print(" Vtune during lock is "); Serial.print(Vtune, 3); Serial.println(" Volts");
      activestate.refVCXOdac = (uint16_t) (4096.0 * Vtune / AREF);
      Serial.print(" new dacsetting is "); Serial.print(activestate.refVCXOdac); Serial.println(" bits\n");
      tcTableSize = 1;                                   //  just do this one time
      setHWDiscipline(  activestate.discipline);         //  update from new setting
      writeStateToEEPROM(activestate);    //   I can't think of a reason not save this for next restart. Should speed things up.  **TODO** Verify this
      Serial.println("\nActive State written to EEPROM/Power-On State");
    }
 
    if (cleanupPLL.checkADF4001locked() & (tcTableSize == 1) &  (abs(activestate.refVCXOT - readPCBTemp() ) > 4.0 ) ) { // fill 2nd  TCtable point when Temp spread is >4C
      if (activestate.vcxoTC == 0) {    //  only calc TC one time since it shouldn't change
        float deltaT =  PCBTemperature - activestate.refVCXOT ;

        int  deltabits ;                                          //  signed intEGER
        uint16_t  bitsnow =  averageBitRead(VTUNE_PIN);
        Serial.print(" ADC bits now are "); Serial.print(bitsnow); Serial.println(" bits");
        float Vtune = ( (float)bitsnow / 4096.0) * calcArefV;
        Serial.print(" Vtune now is "); Serial.print(Vtune, 3); Serial.println(" Volts");
        bitsnow = (uint16_t) (4096.0 * (Vtune / AREF));
        deltabits = bitsnow - activestate.refVCXOdac;
        Serial.print(" deltabits are "); Serial.print(deltabits); Serial.println(" bits");

        activestate.vcxoTC = (float) deltabits / deltaT ;
        Serial.print(" vcxoTC "); Serial.print(activestate.vcxoTC); Serial.println(" bits/degree");
        tcTableSize = 2;

        writeStateToEEPROM(activestate);    //   I can't think of a reason not save this for next restart. Should speed things up.  **TODO** Verify this
        Serial.println("\nActive State written to EEPROM/Power-On State");
      }
     }

    if (activestate.discipline != XO) { // update TCXO if delta-T > [.5]? degree  needs to be evaluated for meas noise
      //buildTcTable(); //  just a stub for now
    }
    rechecktime = millis() + 30000;

    if (activestate.autoPromote && !cleanupPLL.checkADF4001locked()  ) {    // if  lock is preferred but absent
      changeDiscipline( Search);
    }

    myGNSS.checkUblox();                // Check for the arrival of new data and process it. THIS COULD BE PROBLEMMATIC, variable time and sometimes long **TODO**
    myGNSS.checkCallbacks();            // Check if any callbacks are waiting to be processed.
  } //  END (millis() >  rechecktime)

 ///////////////////////////// heartbeat & warning LEDs  ///////////////////////////

  delay(housekeepingdelay);
  fastBlink = !fastBlink;                         //  SW timers for displays 
  if (++ledCounter > 10) {                        //  toggle slowBlink every 10*housekeepingdelay ms
    ledCounter = 0;
    slowBlink = !slowBlink;
  }

  lockModeButNotLocked = ( (discipline == GNSS) | (discipline == Ext10)  ) & !(cleanupPLL.checkADF4001locked()  );
  neededHWnotPresent = (!adf4001Present | !wiFiModulePresent | !si5351Present | !DACPresent  | !tpCapableGNSSPresent);
    
  //  MAY CRASH SPI !!!!!  shared D13 line with SCK, use SPI.begin() to reclaim it
  pinMode(LED_BUILTIN, OUTPUT) ;      //  SPI messes with D13, set it as output so fastBlink will work...

  if(neededHWnotPresent)  {           //  solid IoT led if don't have resources, will require a restart to repair
      digitalWrite(LED_BUILTIN,   true  );
  }
  else if (lockModeButNotLocked)  {   //  slow  blink IoT if lock failure, might repair itself
      digitalWrite(LED_BUILTIN,   slowBlink  ); //  LockLED is connected to MUX on RE so will take care of itself
  }
  else {                              //  just blink the IoT showing normal loop activity
    digitalWrite(LED_BUILTIN,   fastBlink  );               // else solid on warn condition during 
  }

}
//  end loop
