#include "RefGen.h"
//
//
void changeDiscipline(disciplineType discipline) {  //  After calling si5351Setup() select from what we have to discipline the si5351c.
  if ( si5351Present  )  {                          //   si5351present flag, has to be true or don't bother
    //
    if (discipline != Search)  {
      setHWDiscipline(discipline);                  //  explicit request, just do it
    }
    //
    else {                                          //  Search to see if any locked mode is possible
      setHWDiscipline(Ext10);                       //  EXT10 is preferred so use it first if we can
      boolean locked = false;
      int begintime = millis();
      while (!locked && ( (millis() - begintime) < 200) ) {     //  should lock in < 40 ms
        locked = cleanupPLL.checkADF4001locked();
      }
      if (locked) {
        Serial.print ("DEBUG: Best Discipline found is EXT10 in " );
        Serial.print( (millis() - begintime));
        Serial.println(" ms");
      }
      //
      else if  ( tpCapableGNSSPresent && (numSats() > 9) )  {    // We're here because no Ext10 lock, so next checking for GNSS lock
        setHWDiscipline(GNSS);
        locked = false;
        begintime = millis();
        while (!locked && ( (millis() - begintime) < 2000) ) {  //  not sure how much time to give it. seems to lock in under 1 second
          locked = cleanupPLL.checkADF4001locked();
        }
        if ( locked ) {
          Serial.print("DEBUG: Best Discipline found is GNSS lock in ");
          Serial.print( (millis() - begintime));
          Serial.println(" ms");
        }
      } //  end trying GNSS
      //
      if(!locked)  {
        setHWDiscipline( TCXO);
        activestate.discipline = TCXO;
        Serial.println("No Lock");
      }
    } //  end Search
  } // end  si5351Present
} // end changeDiscipline
//
//
void setHWDiscipline( disciplineType newdiscipline) {   //  Force changes to match selected newdiscipline
  switch (newdiscipline) {
    case XO:
      setClock0( freq100M);                            //  move ck0 out of the way for fewer spurs
//      si5351c.output_enable(SI5351_CLK0, 0);

      cleanupPLL.cpdisable = true;                    //  3-state charge pump
      activestate.cpdisable = cleanupPLL.cpdisable;

      setTimepulsePinIsReference(false);              //  enable Satlock detect so line won't be flapping
      
      writeDAC(activestate.refVCXOdac);               //  set the unadjusted tune voltage

      cleanupPLL.pllMux = 7;                          //  force MUX to GND/UnLOCKED
      activestate.pllMux = cleanupPLL.pllMux;         //  keep activestate up-to-date !

      cleanupPLL.ADF4001setHWRegisters();             //  write activestate values to HW

      activestate.discipline = XO;
      break;
    /*************************/
    case TCXO:                                        //  force no locking, but with thermometer & DAC pretune
      setClock0( freq100M);                            //  move ck0 out of the way for fewer spurs
//      si5351c.output_enable(SI5351_CLK0, 0);

      cleanupPLL.cpdisable = true;                    //  3-state charge pump
      activestate.cpdisable = cleanupPLL.cpdisable;   //  keep activestate up-to-date

      setTimepulsePinIsReference(false);              //  enable satLock detect so line won't be flapping
      setTCXOTuneV(readPCBTemp());                    //  set the TCXO tune voltage *adjusted* for temperature.

      cleanupPLL.pllMux = 7;                          //  force MUX to GND
      activestate.pllMux = cleanupPLL.pllMux;         //  keep activestate up-to-date

      cleanupPLL.ADF4001setHWRegisters();             //  write activestate to HW

      activestate.discipline = TCXO;
      break;
    /*************************/
    case Ext10:                                       //  Expects 10M on Ext Input, no timepulse
      setTimepulsePinIsReference(false);              //  turn off any GNSS timepulse on RF_IN_B (p4) that might be present
      setTCXOTuneV(readPCBTemp());                    //  pretune VCXO adjusted for temperature.
      
      si5351c.output_enable(SI5351_CLK0, 1);           //   enable CK0
      setClock0(  freq10M);                             //

      cleanupPLL.referencePinFreq = activestate.fclock0;       //  Si5351 ck0 on REF_IN (pin 8)
      cleanupPLL.rfPinFreq = freq10M;                 //  Ext10  is on RF_IN_A (pin 5)

      cleanupPLL.RCounter = activestate.EXTpfdScaling;   //
      cleanupPLL.NCounter = cleanupPLL.RCounter;      //

      cleanupPLL.pllMux = 1;                          //  set MUX to DLD
      activestate.pllMux = cleanupPLL.pllMux;         //  keep activestate up-to-date

      activestate.cpcurrent = DEFAULT_EXT10CPI;       //
      cleanupPLL.cpcurrent = activestate.cpcurrent;   //  **CHANGED**
      //
      cleanupPLL.cpdisable = false;                   //  enable charge pump
      activestate.cpdisable = cleanupPLL.cpdisable;   //  keep activestate up-to-date

      cleanupPLL.ADF4001setHWRegisters();             //  write activestate to HW
      activestate.discipline = Ext10;
      break;
    /*************************/
    case GNSS:                                        //  force into GNSS disciplining, timpulse pretune
      setTimepulsePinIsReference(true);               //  output timepulse, 4001 p4. NOTE: there may not be any timepulse present !
      setTCXOTuneV(readPCBTemp());                    //  make sure near the correct setting

      si5351c.output_enable(SI5351_CLK0, 1);           //  enable CK0
      setClock0(  UBLOX_MAGIC_FREQ);                    //  CK0 must be 2^N  related to 600/128 

      cleanupPLL.referencePinFreq = activestate.fclock0;       //  These are used for reporting
      cleanupPLL.rfPinFreq = UBLOX_MAGIC_FREQ;        //  importing from Si351 config/MAGICFREQ configured in AAconfig.h
      //
      cleanupPLL.RCounter = activestate.GNSSpfdScaling;
      cleanupPLL.NCounter = cleanupPLL.RCounter;

      cleanupPLL.pllMux = 1;                          //  set MUX to DLD
      activestate.pllMux = cleanupPLL.pllMux;         //

      activestate.cpcurrent = DEFAULT_GNSSCPI;        //
      cleanupPLL.cpcurrent = activestate.cpcurrent;   // 

      cleanupPLL.cpdisable = false;                   //  enable charge pump
      activestate.cpdisable = cleanupPLL.cpdisable;   //  keep activestate up-to-date

      cleanupPLL.ADF4001setHWRegisters();             //  write activestate to HW
      activestate.discipline = GNSS;
      delay(3000);                                    //  wait for loop to settle
      break;
    /*************************/
    default:
      Serial.println("case default discipline, shouldn't be here!!! ");
      break;

  } //  end switch on discipline
  //
  //  Report any changes to discipline type
  if  (activestate.discipline != prevdiscipline) {
    Serial.print("Discipline changed from ");
    Serial.print (getDisciplineName(prevdiscipline) );
    Serial.print(" ===> ");
    Serial.println (getDisciplineName(activestate.discipline) );
    prevdiscipline = activestate.discipline;
  }
 }
//
//
void writeDAC(uint16_t setting) { //  Just write  and don't adjust for T
  extern byte dacbits;            // this value has been defined in myDACsetup()
  if (DACPresent) {
    //          //  Need to adjust write width  based on which 47x6 part
    if (dacbits == 12) {
      myDAC->setVOut(setting);

    }
    if (dacbits == 10) {
      myDAC->setVOut(setting / 4);
    }
    if (dacbits == 8) {
      myDAC->setVOut(setting / 16);
    }

    activestate.dacsetting = setting;
  } // end DACPresent
}
//
//;
void setTCXOTuneV(float temperature) {                                      //   adjusts DAC setting from a calculation or table
  //
  readPCBTemp();                                                            //  call to readPCBTemp updates   PCBTemperature
  float deltaT = PCBTemperature - activestate.refVCXOT ;                      //  T change since  setting dac operating point by default or user
  float tcAdjustment = ( activestate.vcxoTC *  deltaT ) ;                   //  m (bits/delta-degree) referenced to last setting
  activestate.dacsetting = uint16_t (activestate.refVCXOdac + tcAdjustment) ; //   calculate corrected dacsetting at present temperature
  //
  //  Serial.print("tcAdjustment = ");  Serial.println(tcAdjustment );
  //  Serial.print("refVCXOT = ");  Serial.println(activestate.refVCXOT );
  //  Serial.print("refVCXOdac = ");  Serial.println(activestate.refVCXOdac );
  //  Serial.print("tcAdjustment = ");  Serial.println(tcAdjustment );
  //  Serial.print("activestate.dacsetting = ");  Serial.println(activestate.dacsetting );
  //
  writeDAC(activestate.dacsetting);
}
//
uint16_t averageBitRead(int apin) { //  Analog read of any analog input, return average bits
  analogReadResolution(12);         // use full IoT33 ADC resolution,
  analogReference(AR_EXTERNAL);
  uint16_t k = 0;
  float avg = 0;
  uint16_t toss = analogRead(apin);                       //  throw away a zeroeth read to engage the reference
  delay(10);                                              //  and give time for AREF_EXTERNAL to settle
  //
  for (k = 1; k < 101; k++) {     //  average a bunch because it's so fast
    avg += ((float)  analogRead(apin) - avg  ) / k;       //   average[k+1] = avg[k] + ([k+1] - avg[k])/k
    delayMicroseconds(50) ;
  }
  return ( (uint16_t)avg);
}
//
float readPCBTemp() { //  Measures on board sensor and return PCBTemperature in Celsius
  uint16_t br = averageBitRead(THERMOMETER_PIN);
  float mvtemp = ( (calcArefV * 1000)  * ((float) br / 4096.0) )  - 400;          // 400 mv offset @ 0C on tmp236A*
  //  convert to Celsius
  PCBTemperature = (mvtemp / 19.5) ;
  return PCBTemperature;
}
//
//
