#ifndef Reg_h
#define Reg_h
#include <Arduino.h>
class Reg
{
  public:
    /*!
       Constructor
    */
    Reg();
    /*!
       get the current register value
       returns same value as getbf(0,32)
       @return uint32_t value of the register
    */
    uint32_t get()  ;
    /*!
       sets the register value
       @param value uint32_t
    */
    void set(uint32_t value);
    /*!
       current register value

       returns same value as getbf(0,32)
    */
    uint32_t whole ;
    /*!
       modifies the register value based on a value and bitfield (mask)
       @param start index of the bit to start the modification
       @param len length number of bits to modify
       @param value value to modify (value is truncated if larger than len bits)
    */
    void setbf(uint8_t start, uint8_t len , uint32_t value) ;
    /*!
       gets the current register bitfield value, based on the start and length mask
       @param start index of the bit of where to start
       @param len length number of bits to get
       @return bitfield value
    */
    uint32_t getbf(uint8_t start, uint8_t len) ;
};
#endif
