#ifndef  WebServer_h
#define  WebServer_h

#include <Arduino.h>
#include <SPI.h>

#include "AAconfig.h"                   // getting HW and setting constants
#include <WiFiNINA.h>
#include "si5351.h"
#include "ADF4001.h"
#include "ADF4351.h"
#include "RefGen.h"
#include "GNSSTime.h"
#include <Adafruit_SleepyDog.h>

extern void writeStateToEEPROM();

int status = WL_IDLE_STATUS;

extern UBX_NAV_SAT_data_t *ubxDataStruct;

extern void  setClock7(uint32_t fset);
extern void  setClock6(uint32_t fset);
extern void  setClock5(uint32_t fset);
extern void  setClock4(uint32_t fset);
extern void  setClock3(uint32_t fset);
extern void  setClock2(uint32_t fset);
extern void  setClock1(uint32_t fset);
extern void  setClock0(uint32_t fset);

extern disciplineType  discipline, prevdiscipline;
extern void changeDiscipline(disciplineType  discipline);
extern String getDisciplineName( disciplineType discipline);
extern boolean forceTx;
extern void obeyPTT();
extern boolean autoPromote;
extern boolean activeStateHasChangedfromWeb;

extern ADF4001 cleanupPLL;
extern Si5351 si5351c;
extern ADF4351 Synth1;
extern ADF4351 Synth2;

extern void SetSynth1(  uint32_t lo1Freq);
extern void SetSynth2(  uint32_t lo2Freq);

WiFiClient client;
extern uint16_t averageBitRead(uint8_t apin);
extern float readPCBTemp();
extern state activestate;
extern float calcArefV;                    //  best guess at actual ADC Aref, Volts

extern boolean wiFiModulePresent;         // updated by  WebServer
extern boolean tpCapableGNSSPresent;      // updated by GNSSTime
extern boolean adf4001Present;            // updated by ADF4001.checkForHW() (toggle line and see if it moves)
extern boolean si5351Present;             // updated in RefGen
extern boolean DACPresent;                // updated by DAC47x6

//  flags for HW function
extern boolean adf4001Locked ;
extern boolean timePulsePresent;
extern boolean satLocked;

extern boolean synth1Enabled;
extern boolean synth2Enabled;
extern boolean refClockEnabled;
extern uint8_t tcTableSize;
//extern uint32_t fclock1, activestate.fclock2;

extern uint8_t numSVs;                    //    all these extern declarations can be avoided now that clockgen class is extended !
//  Web settable Clock1
void webPrintClkStats();;

extern uint32_t pllB_VCO ;           // PLLB to the nearest Hz
extern uint8_t pllA_a, pllB_a ;           // integer a part of core multiplier
extern uint32_t pllA_b, pllB_b ;          // numerator of fractional part of core multiplier 0-1,048,574cc
extern uint32_t pllA_c, pllB_c ;          // denominator of fractional part of core multiplier, 0-1,048,575
extern uint8_t ck2_a ;                    // integer a part of ck2 divider, (even integers between 6 and 254 inclusive in integer mode)
extern uint32_t ck2_b ;                   // numerator of fractional part of ck2 divider 0-1,048,574
extern uint32_t ck2_c ;                   // denominator of fractional part of ck2 divider, 0-1,048,575extern float ck2_mult ;                        // full ck2 divider a + b/c, between 8 + 1/1,048,575 and 2048

extern uint32_t Ftune;
//
extern void setPreamp(boolean state);
extern void setRxConverter(boolean state);
extern void setTxConverter(boolean state);  
//
//extern BigNumber bigClockFreq(uint32_t refIn,  uint8_t plla, uint32_t pllb, uint32_t pllc, uint8_t diva, uint32_t divb, uint32_t divc,  si5351RDiv_t Rdiv, uint8_t precision );
//extern String actualClockFreq(uint32_t refIn,  uint8_t pa, uint32_t pb, uint32_t pc,  uint8_t da, uint32_t db, uint32_t dc, si5351RDiv_t _Rdiv );  //  prints ref*(a + b/c) 
// export globals .ino may need
boolean WebServersetup();
void WebServerloop();
void manualSetPower(enum si5351_clock clk, uint8_t pwr);

const char* pwrStr[] = {"+4 dBm", "+9 dBm", "+11 dBm", "+13 dBm" }; //  Corresponds to the BOGUS 2,4,6,8 mA of the library
const unsigned int MAX_INPUT = 100;       // max serial data we expect before a newline
uint8_t htmlPageToSend = 0;
int requestedBand = 99;                   //  Preset band tuning

extern boolean resetGNSSModule(); //  For factory reset of GNSS module from Status page
WiFiServer server(port) ;                 // declares server object with internal _port = port
//
#endif
